<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2015 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
namespace xingoauth;

// No direct access
defined('_JEXEC') or die('Restricted access');

/* Load OAuth lib. You can find it at http://oauth.net */
// require_once('OAuth.php');

/**
 * Xing OAuth class
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class XingOAuth
{
	/* Contains the last HTTP status code returned. */
	public $http_code;

	/* Contains the last API call. */
	public $url;

	/* Set up the API root URL. */
	public $host = "https://api.xing.com/v1/";

	/* Set timeout default. */
	public $timeout = 30;

	/* Set connect timeout. */
	public $connecttimeout = 30;

	/* Response format. */
	public $format = 'json';

	/* Decode returned json data. */
	public $decode_json = true;

	/* Whether to return an associative array . */
	public $assoc = false;

	/* Contains the last HTTP headers returned. */
	public $http_info;

	/* Set the useragnet. */
	public $useragent = 'XingOAuth';

	/* Immediately retry the API call if the response was not successful. */
	// public $retry = true;

	/**
	 * __construct
	 *
	 * @param   string  $consumer_key        Param
	 * @param   string  $consumer_secret     Param
	 * @param   string  $oauth_token         Param
	 * @param   string  $oauth_token_secret  Param
	 */
	public function __construct($consumer_key, $consumer_secret, $oauth_token = null, $oauth_token_secret = null)
	{
		$this->sha1_method = new \XTOAuth\OAuthSignatureMethod_HMAC_SHA1;
		$this->consumer = new \XTOAuth\OAuthConsumer($consumer_key, $consumer_secret);

		if (!empty($oauth_token) && !empty($oauth_token_secret))
		{
			$this->token = new \XTOAuth\OAuthConsumer($oauth_token, $oauth_token_secret);
		}
		else
		{
			$this->token = null;
		}
	}

	/**
	 * Set API URLS
	 *
	 * @return  string.
	 */
	public function accessTokenURL()
	{
		return $this->host . 'access_token';
	}

	/**
	 * authenticateURL
	 *
	 * @return  string.
	 */
	public function authenticateURL()
	{
		return $this->host . 'authorize';
	}

	/**
	 * authorizeURL
	 *
	 * @return  string.
	 */
	public function authorizeURL()
	{
		return $this->host . 'authorize';
	}

	/**
	 * requestTokenURL
	 *
	 * @return  string.
	 */
	public function requestTokenURL()
	{
		return $this->host . 'request_token';
	}

	/**
	 * lastStatusCode
	 *
	 * @return  int.
	 */
	public function lastStatusCode()
	{
		return $this->http_status;
	}

	/**
	 * lastAPICall
	 *
	 * @return  string.
	 */
	public function lastAPICall()
	{
		return $this->last_api_call;
	}

	/**
	 * Get a request_token from Xing
	 *
	 * @param   string  $oauth_callback  Param
	 *
	 * @return a key/value array containing oauth_token and oauth_token_secret
	 */
	public function getRequestToken($oauth_callback = null)
	{
		$parameters = array();

		if (!empty($oauth_callback))
		{
			$parameters['oauth_callback'] = $oauth_callback;
		}

		$request = $this->oAuthRequest($this->requestTokenURL(), 'GET', $parameters);
		$token = \XTOAuth\OAuthUtil::parse_parameters($request);

		$oauth_token = null;
		$oauth_token_secret = null;

		if (array_key_exists('oauth_token', $token))
		{
			$oauth_token = $token['oauth_token'];
			$oauth_token_secret = $token['oauth_token_secret'];
		}

		$this->token = new \XTOAuth\OAuthConsumer($oauth_token, $oauth_token_secret);

		return $token;
	}

	/**
	 * Get the authorize URL
	 *
	 * @param   string  $token              Param
	 * @param   string  $sign_in_with_xing  Param
	 *
	 * @return string
	 */
	public function getAuthorizeURL($token, $sign_in_with_xing = true)
	{
		if (is_array($token))
		{
			$token = $token['oauth_token'];
		}

		if (empty($sign_in_with_xing))
		{
			return $this->authorizeURL() . "?oauth_token={$token}";
		}
		else
		{
			return $this->authenticateURL() . "?oauth_token={$token}";
		}
	}

	/**
	 * Exchange request token and secret for an access token and secret, to sign API calls.
	 *
	 * @param   bool  $oauth_verifier  Param
	 *
	 * @return array("oauth_token" => "the-access-token",
	 *                 "oauth_token_secret" => "the-access-secret",
	 *                 "user_id" => "9436992",
	 *                 "screen_name" => "abraham")
	 */
	public function getAccessToken($oauth_verifier = false)
	{
		$parameters = array();

		if (!empty($oauth_verifier))
		{
			$parameters['oauth_verifier'] = $oauth_verifier;
		}

		$request = $this->oAuthRequest($this->accessTokenURL(), 'GET', $parameters);
		$token = \XTOAuth\OAuthUtil::parse_parameters($request);
		$this->token = new \XTOAuth\OAuthConsumer($token['oauth_token'], $token['oauth_token_secret']);

		return $token;
	}

	/**
	 * One time exchange of username and password for access token and secret.
	 *
	 * @param   string  $username  Param
	 * @param   string  $password  Param
	 *
	 * @return array
	 */
	public function getXAuthToken($username, $password)
	{
		$parameters = array();
		$parameters['x_auth_username'] = $username;
		$parameters['x_auth_password'] = $password;
		$parameters['x_auth_mode'] = 'client_auth';
		$request = $this->oAuthRequest($this->accessTokenURL(), 'POST', $parameters);
		$token = \XTOAuth\OAuthUtil::parse_parameters($request);
		$this->token = new \XTOAuth\OAuthConsumer($token['oauth_token'], $token['oauth_token_secret']);

		return $token;
	}

	/**
	 * GET wrapper for oAuthRequest.
	 *
	 * @param   string  $url         Param
	 * @param   array   $parameters  Param
	 *
	 * @return mixed
	 */
	public function get($url, $parameters = array())
	{
		$response = $this->oAuthRequest($url, 'GET', $parameters);

		if ($this->format === 'json' && $this->decode_json)
		{
			return json_decode($response, $this->assoc);
		}

		return $response;
	}

	/**
	 * POST wrapper for oAuthRequest.
	 *
	 * @param   string  $url         Param
	 * @param   array   $parameters  Param
	 *
	 * @return mixed
	 */
	public function post($url, $parameters = array())
	{
		$response = $this->oAuthRequest($url, 'POST', $parameters);

		if ($this->format === 'json' && $this->decode_json)
		{
			return json_decode($response, $this->assoc);
		}

		return $response;
	}

	/**
	 * DELETE wrapper for oAuthReqeust.
	 *
	 * @param   string  $url         Param
	 * @param   array   $parameters  Param
	 *
	 * @return mixed
	 */
	public function delete($url, $parameters = array())
	{
		$response = $this->oAuthRequest($url, 'DELETE', $parameters);

		if ($this->format === 'json' && $this->decode_json)
		{
			return json_decode($response, $this->assoc);
		}

		return $response;
	}

	/**
	 * Format and sign an OAuth / API request
	 *
	 * @param   string  $url         Param
	 * @param   string  $method      Param
	 * @param   array   $parameters  Param
	 *
	 * @return bool
	 */
	public function oAuthRequest($url, $method, $parameters)
	{
		if (strrpos($url, 'https://') !== 0 && strrpos($url, 'http://') !== 0)
		{
			$url = "{$this->host}{$url}";
		}

		$request = \XTOAuth\OAuthRequest::from_consumer_and_token($this->consumer, $this->token, $method, $url, $parameters);
		$request->sign_request($this->sha1_method, $this->consumer, $this->token);

		switch ($method)
		{
			case 'GET':
				return $this->http($request->to_url(), 'GET');
			default:
				return $this->http($request->get_normalized_http_url(), $method, $request->to_postdata());
		}
	}

	/**
	 * Make an HTTP request
	 *
	 * @param   string  $url         Param
	 * @param   string  $method      Param
	 * @param   array   $postfields  Param
	 *
	 * @return bool
	 */
	public function http($url, $method, $postfields = null)
	{
		$this->http_info = array();
		$ci = curl_init();
		/* Curl settings */
		curl_setopt($ci, CURLOPT_USERAGENT, $this->useragent);
		curl_setopt($ci, CURLOPT_CONNECTTIMEOUT, $this->connecttimeout);
		curl_setopt($ci, CURLOPT_TIMEOUT, $this->timeout);
		curl_setopt($ci, CURLOPT_RETURNTRANSFER, true);

		curl_setopt(
			$ci,
			CURLOPT_HTTPHEADER,
			array(
						'Expect:'
			)
		);

		curl_setopt(
			$ci,
			CURLOPT_HEADERFUNCTION,
			array(
				$this,
				'getHeader'
			)
		);

		curl_setopt($ci, CURLOPT_HEADER, false);

		switch ($method)
		{
			case 'POST':
				curl_setopt($ci, CURLOPT_POST, true);

				if (!empty($postfields))
				{
					curl_setopt($ci, CURLOPT_POSTFIELDS, $postfields);
				}
				break;
			case 'DELETE':
				curl_setopt($ci, CURLOPT_CUSTOMREQUEST, 'DELETE');

				if (!empty($postfields))
				{
					$url = "{$url}?{$postfields}";
				}
		}

		curl_setopt($ci, CURLOPT_URL, $url);
		$response = curl_exec($ci);
		$this->http_code = curl_getinfo($ci, CURLINFO_HTTP_CODE);
		$this->http_info = array_merge($this->http_info, curl_getinfo($ci));
		$this->url = $url;
		$curlErrorNum = curl_errno($ci);
		$curlError = curl_error($ci);
		curl_close($ci);

		if ( ($curlErrorNum != CURLE_OK) || (empty($response)) )
		{
			$msg = "HTTP Error: ({$this->http_code}) {$curlError}";

			throw new \Exception($msg);
		}

		return $response;
	}

	/**
	 * Get the header info to store.
	 *
	 * @param   object  $ch      Param
	 * @param   string  $header  Param
	 *
	 * @return bool
	 */
	public function getHeader($ch, $header)
	{
		$i = strpos($header, ':');

		if (!empty($i))
		{
			$key = str_replace('-', '_', strtolower(substr($header, 0, $i)));
			$value = trim(substr($header, $i + 2));
			$this->http_header[$key] = $value;
		}

		return strlen($header);
	}
}
